unit PrintRecord;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, DB,
  DBCtrls, Printers, PrintRecordWait;

type
  THeaderPosition = ( hpBottom, hpLeft, hpRight, hpTop);
  TPrintRecord = class(TComponent)
  private
    FItems : TStrings;
    FDataLink: TFieldDataLink;
    FHeaderFont : TFont;
    FFont : TFont;
    FHeaderPosition : THeaderPosition;
    FDesktopFont: Boolean;
    FParentFont: Boolean;
    FPicture: TPicture;
    function GetDataSource: TDataSource;
    function IsFontStored: Boolean;
    procedure SetFont(Value: TFont);
    procedure SetHeaderFont(Value: TFont);
    procedure SetHeaderPosition(Value: THeaderPosition);
    procedure SetDataSource(Value: TDataSource);
    procedure SetItems(Value: TStrings);
    procedure SetParentFont(Value: Boolean);
    procedure SetDesktopFont(Value: Boolean);
    procedure SetPicture(Value: TPicture);
  protected
    property DesktopFont: Boolean read FDesktopFont write SetDesktopFont default False;
    property ParentFont: Boolean read FParentFont write SetParentFont default True;
  public
    constructor Create(Aowner: TComponent); override;
    destructor Destroy; override;
    procedure Print;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property Header: TStrings read FItems write SetItems;
    property HeaderFont : TFont read FHeaderFont write SetHeaderFont;
    property Font: TFont read FFont write SetFont stored IsFontStored;
    property Picture: TPicture read FPicture write SetPicture;
    property HeaderPosition: THeaderPosition read FHeaderPosition write SetHeaderPosition default hpBottom;
  end;

procedure Register;

implementation

constructor TPrintRecord.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FItems := TStringList.Create;
  FFont := TFont.Create;
  FHeaderFont := TFont.Create;
  FPicture := TPicture.Create;
end;

destructor TPrintRecord.Destroy;
begin
  FPicture.Free;
  FFont.Free;
  FHeaderFont.Free;
  FItems.Free;
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

function TPrintRecord.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TPrintRecord.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

procedure TPrintRecord.Print;
var
  n          : integer;
  CharHeight : Integer;
  Y          : longint;
  HeaderLeft : longint;
  HeaderTop  : longint;
  ImageLeft  : longint;
  ImageTop   : longint;
begin
  Screen.Cursor := crHourGlass;
  Application.CreateForm( TfrmWait, frmWait );
  frmWait.Progressbar.Position := 0;
  frmWait.Show;
  Application.ProcessMessages;
  Printer.BeginDoc;
  Printer.Canvas.Font := HeaderFont;
  CharHeight := Printer.Canvas.TextHeight('Xj') + 4;//calculates the line spacing, based on the header font selected
  Y := CharHeight;
  Case FHeaderPosition of
    hpTop: begin
             HeaderLeft := 10;
             HeaderTop := Y;
             ImageLeft := 10;
             ImageTop := (Header.Count + 1) * CharHeight;
           end;
    hpRight: begin
              HeaderLeft := Picture.Width + 10;
              HeaderTop := Y;
              ImageLeft := 10;
              ImageTop := Y;
            end;
    hpBottom: begin
                HeaderLeft := 10;
                HeaderTop := Picture.Height + CharHeight;
                ImageLeft := 10;
                ImageTop := Y;
              end;
    hpLeft: begin
                HeaderLeft := 10;
                HeaderTop := Y;
                ImageLeft := Printer.PageWidth - Picture.Width - 10;
                ImageTop := Y;
             end;
  end;
  if Assigned( Picture ) then
    Printer.Canvas.Draw( ImageLeft, ImageTop, Picture.Bitmap );
  Application.ProcessMessages;
  Y := HeaderTop;
  For n := 0 to Header.Count - 1 do //Prints the header
  begin
    Printer.Canvas.TextOut( HeaderLeft, Y, Header[n] );
    Y := Y + CharHeight;
  end;
  //if the header is positioned side by side with the picture, must determine
  //which one have the greatest Height, to print next line in the correct
  //position, avoiding overprints -- See the if statement below
  if (FHeaderPosition <> hpBottom) and (FHeaderPosition <> hpTop) then
  begin
    if (Header.Count * CharHeight) < Picture.Height then
      Y := Picture.Height + CharHeight
    else
      Y := (Header.Count + 1) * CharHeight;
  end
  else
    Y := Picture.Height + ((Header.Count + 1) * CharHeight);
  Y := Y + CharHeight;
  Printer.Canvas.Font := Font;
  CharHeight := Printer.Canvas.TextHeight('Xj') + 4; //now calculates the line spacing, based on the Fields font
  frmWait.ProgressBar.Max := DataSource.DataSet.FieldCount;
  For n := 0 to DataSource.DataSet.FieldCount - 1 do
  begin
    if DataSource.DataSet.Fields[n].Visible then  //Prints only the visible fields
    begin
      Printer.Canvas.TextOut( 10, Y, DataSource.DataSet.Fields[n].DisplayName + ': ' +
        DataSource.DataSet.Fields[n].AsString );
      Y := Y + CharHeight;
      frmWait.ProgressBar.Position := n;
      Application.ProcessMessages;
    end;
  end;
  Printer.EndDoc;
  frmWait.Close;
  Screen.Cursor := crDefault;
end;

procedure TPrintRecord.SetItems(Value: TStrings);
begin
  Header.Assign(Value);
end;

procedure TPrintRecord.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
end;

procedure TPrintRecord.SetHeaderFont(Value: TFont);
begin
  FHeaderFont.Assign(Value);
end;

function TPrintRecord.IsFontStored: Boolean;
begin
  Result := not ParentFont and not DesktopFont;
end;

procedure TPrintRecord.SetParentFont(Value: Boolean);
begin
  if FParentFont <> Value then
    FParentFont := Value;
end;

procedure TPrintRecord.SetDesktopFont(Value: Boolean);
begin
  if FDesktopFont <> Value then
    FDesktopFont := Value;
end;

procedure TPrintRecord.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TPrintRecord.SetHeaderPosition(Value: THeaderPosition);
begin
  if FHeaderPosition <> Value then
    FHeaderPosition := Value;
end;

procedure Register;
begin
  RegisterComponents('Data Controls', [TPrintRecord]);
end;

end.
